/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Yahoo! Mail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Michael Kraft.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function ym_notifier() {

  // default constants
  this.defaultloginURL     = "https://login.yahoo.com";
  this.btloginURL          = "https://edit.europe.yahoo.com";

  this.loginPATH           = "/config/login";
  this.verifyPATH          = "/config/login_verify2";
  this.logoutPATH          = "/config/login?logout=1";

  this.defaultcheckURL     = "http://www.yahoo.com";
  this.altcheckURL	       = "http://uk.update.companion.yahoo.com/slv/v4/not";
  
  // The first url below will reset the new mail count when it gets the folder unread count, the second one won't
//  this.mailFolderUnreadURL       = "http://us.mg1.mail.yahoo.com/ws/mail/v1/formrpc?m=ListFolders&appid=YahooMailRC&resetMessengerUnseen=true"
  this.mailFolderURL       = "http://us.mg1.mail.yahoo.com/ws/mail/v1/formrpc?m=ListFolders&appid=YahooMailRC"

  this.defaultmailURL      = "http://mail.yahoo.com";
  this.defaultcookieDomain = "yahoo.com";
  
  // these can change based of user's email
  this.loginURL            = this.defaultloginURL;
  this.checkURL            = this.defaultcheckURL;
  this.mailURL             = this.defaultmailURL;
  this.cookieDomain        = this.defaultcookieDomain;

  // preferences
  this.prefs = new ym_prefs();
  this.prefs.initPrefs();
  
  // Delete log file if not logging
  try {
    if (!this.prefs.getPref(this.prefs.PREF_LOG_ENABLED)) {
      var logFile = Components.classes["@mozilla.org/file/directory_service;1"].
                    getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsILocalFile).clone();
		  logFile.append("Yahoo_Mail_Notifier_log.txt");
		  if (logFile.exists()) {
  		  logFile.remove(false);
		  }
	  }
	}
	catch (ex) {}
  
  // current password for default user
  this.currentDefaultPassword = null;
  
  // Is this an older browser based off of buggy FF 1.0.7 code?
  this.oldBrowser = true;        // default to true
  
  this.loginFailCount = 0;
  this.newMailCount = 0;
  this.doNotCheck = true;        // windows default to not checking
  this.useAltCheck = false;      // Use the alternate checking server, defaults to false. 

  this.mouseEvtTime = 0;         // save time of click event.
  this.mouseId = null;           // handle of click setTimeOut
  this.timer = null;             // NsiTimer object
  this.sound = null;             // NsiSound object
  this.sendRetryId = null;       // handle http retry setTimeout for login
  this.sendRetry2Id = null;      // handle http retry setTimeout for checkmail
  this.sendRetry3Id = null;      // handle http retry setTimeout for getMailFolderCount

  this.folderFailCount = 0;      // Fail count when checking folders (try 3 times);

  // 
  this.xmlHttpReq = null;        // XMLHttpRequest object
  this.logFile = null;           // Log File
  this.mEOL = this.getEOL();     // EOL for log file (based on O/S)

  // NsiObserver object
  this.observer = {
    observe : function(aSubject, aTopic, aData) {
      gYm_notifier.observe(aSubject, aTopic, aData);
    }
  }
}

/*
 * do stuff when new browser window opens 
 * use a settimeout to allow window to open if masterpassword is set
 */
ym_notifier.prototype.initialize = function() {
  if (document.getElementById("ym-notifier-statusbar-item")) { 
    window.addEventListener('unload', gYm_notifier.uninitialize, false);
    window.setTimeout("gYm_notifier.initialize2()", 5);
  }
  window.removeEventListener('load', gYm_notifier.initialize, false);
}

ym_notifier.prototype.initialize2 = function() {
  var prefs = this.prefs;
  var showStatusbar = (prefs.getPref(prefs.PREF_STATUSBAR) == prefs.CONST_ALWAYS);
  var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
  if (myStatusbar) myStatusbar.setAttribute("collapsed", !showStatusbar);

  // Firefox 1.0.7 based browsers suffer from bug 
  // https://bugzilla.mozilla.org/show_bug.cgi?id=174320
  // so we need to know if this is an older browser or not.  linkedBrowser
  // only exists in browsers based off the FF 1.5 and SeaMonkey code.
  try {
    if (document.getElementById("content").mCurrentTab.linkedBrowser) this.oldBrowser = false;
  } catch (ex) {}
  this.ym_log("Old Browser = " + this.oldBrowser);
    
  try {
    // Add observer for when logging in and out and for changing new mail indicator
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(gYm_notifier.observer,"ym-gui-change",false);   
    prefs.addObserver(prefs.PREF_STATUSBAR, gYm_notifier.observer);

    // Is this the first browser window?
    var xulWindow;
    var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].
             getService(Components.interfaces.nsIWindowMediator);
    var browsers=wm.getXULWindowEnumerator('navigator:browser');
    if (browsers.hasMoreElements()) { xulWindow=browsers.getNext(); }

    // This is the first window
    if (!browsers.hasMoreElements()) {
      this.ym_log("First window");
      	
      // allow this window to check
      this.doNotCheck = false;
      
      // initialize XMLHttpRequest Object
      this.xmlHttpReq = new XMLHttpRequest();
      	
      // listen for interval changes and start checking for email if set to do so
      prefs.addObserver(prefs.PREF_INTERVAL, gYm_notifier.observer);
        
      if (prefs.getPref(prefs.PREF_AUTO_CHECK)) {
      	var loginState = this.loginState();
        this.ym_log("loginState : " + loginState);
        if (loginState < 4) {
          this.doLogin();
        }
        else {
          this.startChecking();
        }
      }
    }
    else {
      this.ym_log("Not first window");

      xulWindow = xulWindow.QueryInterface(Components.interfaces.nsIXULWindow);

      // set icons, menus and tooltips to the same as other windows
      var masterStatusbar = xulWindow.docShell.contentViewer.DOMDocument
                            .getElementById("ym-notifier-statusbar-item");

      // get new mail count from tooltip
      if (masterStatusbar) {
      	var messageCount = masterStatusbar.getAttribute("label");
      	var lastCheckTime = masterStatusbar.getAttribute("lastCheckTime"); 
      	if (masterStatusbar.getAttribute("error") == "true") this.updateGUI("check-error");
      	else if (messageCount) this.updateGUI(messageCount + "  " + lastCheckTime);
      	else this.updateGUI("0  " + lastCheckTime);
      }
    }
  }
  catch (ex) {
    dump(ex);
    this.ym_logError(ex, 1);
  }
}
   
/* 
 * do stuff when browser window closes
 */
ym_notifier.prototype.uninitialize = function() {
  try {
    gYm_notifier.ym_log("Uninitializing");
    // Remove observers
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(gYm_notifier.observer,"ym-gui-change");   
    gYm_notifier.prefs.removeObserver(gYm_notifier.prefs.PREF_STATUSBAR, gYm_notifier.observer);

    var startNewTimer = false;

    // If this window has a timer, stop it (should only happen for polling window)
    if (gYm_notifier.timer) {
      gYm_notifier.prefs.removeObserver(gYm_notifier.prefs.PREF_INTERVAL, gYm_notifier.observer);
      gYm_notifier.stopChecking();
      startNewTimer = true;
    }
    
    // Is this window polling, if so get another window to do it
    // A component would be better, do that later
    if (!gYm_notifier.doNotCheck) {
      // Make sure request object is gone
      delete(gYm_notifier.xmlHttpReq);
      gYm_notifier.xmlHttpReq = null;
    
      // Find another window to take over polling job
      var newYmNotifier = null;
      var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].
               getService(Components.interfaces.nsIWindowMediator);
      var browsers=wm.getXULWindowEnumerator('navigator:browser');
      if(browsers.hasMoreElements()) { 
        var xulWindow = browsers.getNext();
        xulWindow = xulWindow.QueryInterface(Components.interfaces.nsIXULWindow);
        var requestor = xulWindow.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
        var win = requestor.getInterface(Components.interfaces.nsIDOMWindow);
        newYmNotifier = win.gYm_notifier;
        newYmNotifier.prefs.addObserver(newYmNotifier.prefs.PREF_INTERVAL, newYmNotifier.observer);
      
        // Allow new window to check
        newYmNotifier.doNotCheck = false;
      
        // initialize XMLHttpRequest Object
        newYmNotifier.xmlHttpReq = new XMLHttpRequest();
      	
        // Copy current default password to new window
        newYmNotifier.currentDefaultPassword = gYm_notifier.currentDefaultPassword;
        
        // Copy current mail count to new window
        newYmNotifier.newMailCount = gYm_notifier.newMailCount;
      }
      // last window closing, check to see if user does not want to remember password and if so logout
      // to remove yahoo cookie
      else {
        // TBD MULTI-USER SUPPORT
        var username = gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_DEFAULT_USER);
        if (username) {
          var userInfo = gYm_notifier.prefs.loadUser(username, false);
          if (userInfo && !userInfo.remember_pw) gYm_notifier.logout(username);
        }
      }
    }
    // Tell new window to start checking for mail but to not actually check at this moment. The reason 
    // for this is that if checkNow is called from this window,  the XMLHttpRequest component 
    // will generate an error.  I do not know why this happens, but it does so just don't call it.
    // This may cause a slight delay in checking for mail.  Once the component is implemented this 
    // won't be a problem.
    if (newYmNotifier && startNewTimer) newYmNotifier.startChecking(true);
    
    window.removeEventListener('unload', gYm_notifier.uninitialize, false);
  }
  catch (ex) {
    gYm_notifier.ym_logError(ex, 1);
  }
  finally {
    // remove instance to ourselves
    gYm_notifier = null;
  }
}
   
/*
 * start checking for new mail
 */
ym_notifier.prototype.startChecking = function(dontCheckNow) {
  this.ym_log("begin startChecking");
	
  if (this.doNotCheck) {
    this.ym_log("Window not allowed to check");
    return;
  }

  if (this.timer) {
    this.timer.cancel();
    this.ym_log("restarting timer");
  }
  // this needs to be done even if the timer is cancel because of bug 325418 otherwise time will be off
  // if user changes interval
  // https://bugzilla.mozilla.org/show_bug.cgi?id=325418
  this.timer = Components.classes["@mozilla.org/timer;1"].
               createInstance(Components.interfaces.nsITimer);

  this.timer.init(gYm_notifier.observer, this.prefs.getPref(this.prefs.PREF_INTERVAL) * 60000, 
                  Components.interfaces.nsITimer.TYPE_REPEATING_PRECISE);
 
  // By default, check now otherwise it will start checking after the timer expires
  if (!dontCheckNow) {
  
    // Firefox 1.0.7 based browsers suffer from https://bugzilla.mozilla.org/show_bug.cgi?id=174320
    // Work around is to use settimeout
    if (this.oldBrowser) window.setTimeout("gYm_notifier.checkMail();", 0)
    else gYm_notifier.checkMail();
  }
 
  this.ym_log("leave startChecking");
}

/*
 * stop checking for new mail
 */
ym_notifier.prototype.stopChecking = function() {
  this.ym_log("stopChecking");
  if (this.timer) {
    this.ym_log("timer canceled");
    this.timer.cancel();
    this.timer = null;
  }
  if (this.xmlHttpReq) this.xmlHttpReq.abort();
}

/*
 * log out of Yahoo! by deleting cookie
 */
ym_notifier.prototype.logout = function(username) {
  this.ym_log("logging out");
  try {
    // set correct URLs
    this.findYahooURL(username);
  	
    // logout by deleting login cookie
    var cookMan = Components.classes["@mozilla.org/cookieService;1"].
                  getService(Components.interfaces.nsICookieManager);
    cookMan.remove("." + this.cookieDomain, "Y", "/", false);

    // if default user forget default password   // TBD MULTI-USER SUPPORT
    if (username == this.prefs.getPref(this.prefs.PREF_DEFAULT_USER))
      this.currentDefaultPassword = null;

    // clear stored cookie
    this.prefs.saveUserCookie("", username);

    // notify all windows of logout
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.notifyObservers(null,"ym-gui-change","logout " + username);
  }
  catch (ex) {
    this.ym_logError(ex, 1);
  }
}

/*
 * Manual check for email or login if not already logged in
 */
ym_notifier.prototype.manualCheckMail = function () {
  var loginState = gYm_notifier.loginState();
  this.ym_log("manualCheckMail\nloginState : " + loginState);
  if (loginState < 4) {
    gYm_notifier.doLogin()
  }
  else {
    gYm_notifier.checkMail();
  }
}

/*
 * Find correct URL to use if user specifies email address
 * Currently this is needed only for BT Yahoo and Yahoo Japan as 
 * they are the only two which won't work.  Ignore the email for others
 * since it will break.  Yahoo will allow the user to login with their
 * email address so that can be used as the login name.
 */
 
ym_notifier.prototype.findYahooURL = function(username) {
	
  // if there is not a defined username use the default one  // TBD MULTI-USER SUPPORT
  if (!username) username = this.prefs.getPref(this.prefs.PREF_DEFAULT_USER);

  // set default URLS
  this.cookieDomain = this.defaultcookieDomain;
  this.mailURL      = this.defaultmailURL;
  this.loginURL     = this.defaultloginURL;
  this.checkURL     = (this.prefs.getPref(this.prefs.PREF_ALT_CHECK))? this.altcheckURL : this.defaultcheckURL;

  // find @ and ., if this is found its an email address
  var atSymbol = username.indexOf("@");
  var lastdot = username.lastIndexOf(".");

  // If companion UK server failed use the US one since either will work
  if (this.prefs.getPref(this.prefs.PREF_ALT_CHECK) && this.useAltCheck) {
    this.checkURL =this.altcheckURL.replace("http://uk.","http://");
  }
	
  // if username is actually an email address, strip off the username and find the login server
  if ((atSymbol != -1) && (lastdot != -1)) {

    // This should handle the case of BT Yahoo
    if (this.prefs.getPref(this.prefs.PREF_ALT_CHECK) && (username.indexOf("@btinternet.com") != -1)) {
      this.loginURL     = this.btloginURL;
      this.mailURL = this.mailURL.replace("http://","http://uk.");
      this.checkURL = this.altcheckURL;
    }
    // Yahoo! Japan uses different URLS
    if (username.indexOf("@yahoo.co.jp") != -1) {    	
      this.cookieDomain = "yahoo.co.jp";
      this.loginURL     = "https://login.yahoo.co.jp";
      this.checkURL     = "http://update.toolbar.yahoo.co.jp/slv/v4/not";
      this.mailURL      = "http://mail.yahoo.co.jp";
    }
  }
  
  //this.ym_log("loginURL = " + this.loginURL + ", checkURL = " + this.checkURL + ", mailURL = " + 
  //            this.mailURL + ", cookieDomain = " + this.cookieDomain);
}

/* 
 * Login (if necessary) and begin polling
 */
ym_notifier.prototype.doLogin = function() {
  this.ym_log("doLogin");

  // A better way would be to observe changes to the Yahoo cookies and automatically clear
  // the stored cookie if user logs out at web site, but this works for now.  A minor problem
  // is that Yahoo will keep asking to verify the password even though we are logged in.  
  // Observing when the new cookie is stored in the browser and storing it into the user 
  // would lessen this problem.  Really we should "verify" the login at least once every
  // 24 hours.
  // The current work around is to "forget" the login cookie if over 12 hours has passed
  // since the user last actually logged (didn't use cookie)

  // TBD MULTI-USER SUPPORT
  var username = this.prefs.getPref(this.prefs.PREF_DEFAULT_USER);
  if (username) {
    var userInfo = this.prefs.loadUser(username);
    if (userInfo) {
      var password = userInfo.password; 
      var thecookie = userInfo.cookie;
      var useCookie = userInfo.useCookie;
      delete userInfo; 

      var loginState = this.loginState(username);
      
      // set correct URLs
      this.findYahooURL(username);

      // only log in using cookies, if not logged in as someone else 
      if (useCookie && thecookie && (loginState != 1) &&
          this.getCookie("." + this.cookieDomain, 'T')) {
        this.loginWithCookie(username, thecookie);
        return;
      }
      else if (password) {
        this.login(username, userInfo.password);
        return;
      }
      // temporarily stored password for default user
      else if (this.currentDefaultPassword) {
        this.login(username, this.currentDefaultPassword);
        return;
      }
    }
  }
  // either no user or no password so open login window
  this.openLoginWindow();
}

/*
 * log in to Yahoo! using stored cookie info
 */
ym_notifier.prototype.loginWithCookie = function(username, cookieValue) {

  this.ym_log("Logging in with cookie " + cookieValue);

  var ioService = Components.classes['@mozilla.org/network/io-service;1'].
                  getService(Components.interfaces.nsIIOService);
  var cookieService = Components.classes["@mozilla.org/cookieService;1"].
                      getService(Components.interfaces.nsICookieService);
  var observerService = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
  
  // set correct URLs
  this.findYahooURL(username);
  	
  var uri = ioService.newURI("http://" + this.cookieDomain, null, null);
  var cookieString = "Y=" + cookieValue + ";domain=." + this.cookieDomain + 
                     ";path=/;expires=Thursday, April 15, 2010 3:57:58 PM;"
  cookieService.setCookieString(uri, null, cookieString, null);

  observerService.notifyObservers(null,"ym-gui-change","login " + username);
  gYm_notifier.loginFail = false;
}

/* 
 * log in to Yahoo! if necessary
 */
ym_notifier.prototype.login = function(username, pword, persist, challenge, challengeTry) {
  this.ym_log("login start");
	
  // set correct URLs
  this.findYahooURL(username);

  var theUserName = username;
  var loginState = this.loginState(username);
  var theURL = this.loginURL + this.loginPATH;
  var persistent = "&.persistent=Y";
  var challengeParam = "";

  if (persist==false || loginState == 2 || loginState == 4) persistent = "";
  if (challenge && challenge != "undefined") challengeParam = "&.challenge=" + challenge;

  // find @ and ., if this is found its an email address
  var atSymbol = username.indexOf("@");
  var lastdot = username.lastIndexOf(".");
  
  // if username is actually an email address, check for partners
  if ((atSymbol != -1) && (lastdot != -1)) {

    // Yahoo! Japan won't work if username is email address
    if (username.indexOf("@yahoo.co.jp") != -1) 
      theUserName = username.substring(0,atSymbol);
  }

  this.ym_log("Logging in as " + theUserName + " with persist = " + persist + 
              " and challenge = " + challenge);

  if (loginState == 1) {
    this.ym_log("Logging out first because logged in as wrong user.");
    this.logout(username);
  }
  else if (loginState > 1) {
    theURL = this.loginURL + this.verifyPATH;
    this.ym_log("User was already logged in, verifying login.");
  }
  this.ym_log("Login URL: " + theURL);

  try {
    // The request object should always exist, but just in case...
    if (!this.xmlHttpReq) {
      this.ym_log("xmlHttpReq not created yet, so creating it.");
      this.xmlHttpReq = new XMLHttpRequest();
    }
    this.xmlHttpReq.open("POST", theURL, true);
    this.xmlHttpReq.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    this.xmlHttpReq.setRequestHeader('Cache-Control','no-cache');
    this.xmlHttpReq.setRequestHeader('Pragma','no-cache');
    this.xmlHttpReq.onerror = function() {
      gYm_notifier.ym_log("login Onerror fired.");
      gYm_notifier.xmlHttpReq.abort();
      // if login was trigger automatically (with no user action) it must have been while 
      // checking so change the icon and put up error tooltip, otherwise put up alert window
      if (gYm_notifier.timer) {
        var observerService = Components.classes["@mozilla.org/observer-service;1"].
                              getService(Components.interfaces.nsIObserverService);
        observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
      }
      else alert('"' + username + '" - ' + gYm_notifier.getString("loginfail")); 
    }
    this.xmlHttpReq.onload = function() {
      if (gYm_notifier.xmlHttpReq.readyState==4) {
        if (gYm_notifier.xmlHttpReq.status == 200) {
          gYm_notifier.loginFailCount = 0;
          var observerService = Components.classes["@mozilla.org/observer-service;1"].
                                getService(Components.interfaces.nsIObserverService);
          if (gYm_notifier.xmlHttpReq.responseText.indexOf('window.location.replace') != -1) {
            // notify all windows of login
            observerService.notifyObservers(null,"ym-gui-change","login " + username);
            gYm_notifier.loginFail = false;
            // save password if this was default user  // TBD MULTI-USER SUPPORT
            if (username == gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_DEFAULT_USER)) {
              gYm_notifier.currentDefaultPassword = pword;
            }
          }
          // BT Internet needs a challenge parameter so this is the work around.
          // Sometimes it doesn't work so we try twice.
          else if (!challengeParam || challengeParam=="undefined" || challengeTry=="first") {
            var checkText = '".challenge" value="';
            var tryText = "first";
            if (challengeTry=="first") tryText = "second";
            var sChallenge = gYm_notifier.xmlHttpReq.responseText.indexOf(checkText);
            if (sChallenge != -1) {
              var eChallenge = gYm_notifier.xmlHttpReq.responseText.indexOf
                               ('"', sChallenge + checkText.length);
              if (eChallenge != -1) {
                var challengeText = gYm_notifier.xmlHttpReq.responseText.substring
                                 (sChallenge + checkText.length, eChallenge);
                gYm_notifier.ym_log("login failed, trying " + tryText + " challenge = " + challengeText);
                // Firefox 1.0.x requires the following line or it won't work for some reason
                if (gYm_notifier.oldBrowser) gYm_notifier.xmlHttpReq = null;
                gYm_notifier.login(username, pword, persist, challengeText, tryText);
              }  
            }
          }
          else {
            alert ('"' + username + '" - ' + gYm_notifier.getString("baduser"));
            gYm_notifier.loginFail = true;
            observerService.notifyObservers(null,"ym-gui-change","logout " + username);
          }
        }
        else {
          gYm_notifier.loginFailCount = gYm_notifier.loginFailCount + 1;
          if (gYm_notifier.loginFailCount > 3) {
            gYm_notifier.loginFailCount = 0;
            // if login was trigger automatically (with no user action) it must have been while 
            // checking so change the icon and put up error tooltip, otherwise put up alert window
            if (gYm_notifier.timer) {
              var observerService = Components.classes["@mozilla.org/observer-service;1"].
                                    getService(Components.interfaces.nsIObserverService);
              observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
            }
            else alert('"' + username + '" - ' + gYm_notifier.getString("loginfail"));
            gYm_notifier.xmlHttpReq.abort();
          }
          else {
            gYm_notifier.login(username, pword);
          }
        }
      } 
    }
    // If xmlHttpReq is ready, send request
    if (this.xmlHttpReq.readyState == 1) {
      // if retrying clear pending retry
      if (this.sendRetryId) {
        clearTimeout( this.sendRetryId );   // Clear pending retry  
        this.sendRetryId = null;
      }    	
    	
      // Send the request, even though done is set to this.defaultcheckURL it doesn't really
      // make a difference as long as it is a valid URL
      this.xmlHttpReq.send('.tries=1&hasMsgr=0&.v=0&.chkP=Y&.done=' + this.defaultcheckURL + 
                            persistent + '&login=' + encodeURIComponent(theUserName) + 
                            '&passwd=' + encodeURIComponent(pword) + challengeParam);
    }
    // if it is not ready try again in 2 seconds
    else {
      this.ym_log("Login readyState = " + this.xmlHttpReq.readyState + ", retrying in 2 seconds.");
      this.xmlHttpReq.abort();
      this.sendRetryId = setTimeout("gYm_notifier.login(username, pword, persist, challenge, challengeTry);", 2000);
    }
  }
  catch (ex)
  {
    this.ym_logError(ex, 1);
  }
}
   
/*
 * Grabs the list of folders and how much unread and total mail there is in the mailbox.  Not currently used.
 * If the this.mailFolderUnreadURL is used, it will also reset the new mail count when checking.
 * This currently isn't used at the moment.
 */
ym_notifier.prototype.getMailFolderCount = function(username, aURL) { 
  // Doesn't work for Yahoo Japan.
  if (username.indexOf("@yahoo.co.jp") != -1) return;
  var theURL = (aURL) ? aURL : this.mailFolderURL;
  this.ym_log('Trying to clear new mail count at ' + theURL);
  try {
    // Create a new XMLHttpRequest object so it doesn't interfere with checking for mail.
    var requester = new XMLHttpRequest();
    requester.open("GET", theURL, true);
    requester.setRequestHeader('Cache-Control','no-cache');
    requester.setRequestHeader('Pragma','no-cache');
    requester.onerror = function() {
      // An error occurred, notify user
      gYm_notifier.folderFailCount = gYm_notifier.folderFailCount + 1;
      gYm_notifier.ym_log("getMailFolderCount Onerror fired. Failed " + gYm_notifier.folderFailCount + " times." +
                          ((gYm_notifier.folderFailCount>3)?"Giving Up":"Trying again in 2 seconds"));
      this.abort();
      if (gYm_notifier.folderFailCount > 3) {
        gYm_notifier.folderFailCount = 0;
      }
      else {
        this.sendRetry3Id = setTimeout("gYm_notifier.getMailFolderCount();", 2000);
      }
    }
    requester.onload = function() {
      gYm_notifier.ym_log('getMailFolderCount request status (' + this.status + ')');
      if (this.readyState==4) {
        gYm_notifier.ym_log("ReponseText: " + this.responseText);
        gYm_notifier.folderFailCount = 0;
        if (this.status == 200) {
          var xmldoc = this.responseXML;
          var folders = xmldoc.getElementsByTagName('folder');
          var unread = 0;
          var total = 0;
          for (var i=0; i<folders.length; i++) {
            if ((folders.item(i).getAttribute("isSystem") == "false") || (folders.item(i).firstChild.getAttribute("name") == "Inbox")) {
			        unread = unread + parseInt(folders.item(i).getAttribute("unread"));
			        total = total + parseInt(folders.item(i).getAttribute("total"));
		        }
		      }
		      gYm_notifier.ym_log("Folders: " + folders.length + ", Unread: " + unread + ", Total: " + total);
        }
        else if (this.status == 500) {
          var xmldoc = this.responseXML;
          var newURL = xmldoc.getElementsByTagName('url').item(0);
          if ((typeof(eval(newURL)) != "undefined") && newURL && newURL.firstChild) {
            gYm_notifier.ym_log("Redirection URL: " + newURL.firstChild.data);
            gYm_notifier.getMailFolderCount(username, newURL.firstChild.data);
          }
        }
        else {
          var xmldoc = this.responseXML;
          var reasonText = "none";
          if (xmldoc) {
             var message = xmldoc.getElementsByTagName('message').item(0);
             if ((typeof(eval(message)) != "undefined") && message && message.firstChild) reasonText = message.firstChild.data;
          }
          gYm_notifier.ym_log("Could not retrieve folder count. Reason: " + reasonText);
        }
      }
    }
    // If xmlHttpReq is ready, send request
    if (requester.readyState == 1) {
      // if retrying clear pending retry
      if (this.sendRetry3Id) {
        clearTimeout( this.sendRetry3Id );   // Clear pending retry  
        this.sendRetry3Id = null;
      }    	
      requester.send(null);
    }
    // if it is not ready try again in 2 seconds
    else {
      this.ym_log("getMailFolderCount readyState = " + requester.readyState + ", retrying in 2 seconds.");
      requester.abort();
      this.sendRetry3Id = setTimeout("gYm_notifier.getMailFolderCount();", 2000);
    }
  }
  catch (ex)
  {
    this.ym_logError(ex, 1);
  }
}
 
/*
 * check for new mail
 */   
ym_notifier.prototype.checkMail = function() {

  // Check to see if the same user is still logged in
  // TBD MULTI-USER SUPPORT
  var loginState = this.loginState();
  if (loginState <= 1) {
     // logout to prevent problems
     this.logout();
     
     // there should probably be some kind of alert here to tell
     // user that the extension logged back into default user.
  }
	
  // Check if its time to relogin to Yahoo before checking for mail
  // TBD MULTI-USER SUPPORT
  var userInfo = gYm_notifier.prefs.loadUser("", false);
  var username = null;
  if (userInfo) {
     var useCookie = userInfo.useCookie;
     username = userInfo.userName;
     delete userInfo;
     if (!useCookie) {
       this.ym_log('Time to relogin to Yahoo')
       this.doLogin();
       return;
     }
  }

  // set correct URLs
  this.findYahooURL(username);
  	
  this.ym_log('Checking mail for ' + username + ' at ' + this.checkURL)
  
  // The request object should always exist, but just in case...
  if (!this.xmlHttpReq) {
    this.ym_log("xmlHttpReq not created yet, so creating it.");
    this.xmlHttpReq = new XMLHttpRequest();
  }
  this.xmlHttpReq.open("GET", this.checkURL, true);
  this.xmlHttpReq.setRequestHeader('Cache-Control','no-cache');
  this.xmlHttpReq.setRequestHeader('Pragma','no-cache');
  this.xmlHttpReq.onerror = function() {
    // An error occurred, notify user
    gYm_notifier.ym_log("checkmail Onerror fired.");
    gYm_notifier.xmlHttpReq.abort();
    var observerService = Components.classes["@mozilla.org/observer-service;1"].
                          getService(Components.interfaces.nsIObserverService);
    observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
    
    // if using companion swerver swap check servers in case one server is having issues
    if (gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_ALT_CHECK)) gYm_notifier.useAltCheck = !gYm_notifier.useAltCheck;
  }
  this.xmlHttpReq.onload = function() {
    gYm_notifier.ym_log('request status (' + gYm_notifier.xmlHttpReq.status + ')');
    if (gYm_notifier.xmlHttpReq.readyState==4) {
      if (gYm_notifier.xmlHttpReq.status == 200) {
        var response = gYm_notifier.xmlHttpReq.responseText;
        var result = null;
        if (gYm_notifier.checkURL == gYm_notifier.defaultcheckURL) {
          gYm_notifier.ym_write_log('Result: ' + response + '\n');
          if (/id=\"pamail\".*<strong>(\d+)<\/strong>/.test(response)) result = RegExp.$1;
          gYm_notifier.ym_log('Result: ' + result);
        }
        else if (gYm_notifier.checkURL == "http://update.toolbar.yahoo.co.jp/slv/v4/not") {
          gYm_notifier.ym_log('Response: ' + response);
          if (/\s(\d+)\s.*yahoo/.test(response))  result = RegExp.$1;
          gYm_notifier.ym_log('Result: ' + result);
        }
        else if (gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_ALT_CHECK)) {
          gYm_notifier.ym_log("Response: " + response);
          if (/\s(\d+)[\s+].*(http:\/\/[(a-z)+\.]+\.yahoo[\.(a-z)+]+).*poll/m.test(response)) result = RegExp.$1;
          // Check for weird HK 1 mail check result
          if (!result) {
            result = /@.*(http:\/\/[(a-z)+\.]+\.yahoo[\.(a-z)+]+).*poll/m.test(response);
            if (result && RegExp.$1 == "http://hk.mail.yahoo.com") {
              result = "1";
            }
            else result = null;
          }
        }

        var now = new Date();
        var hours = now.getHours();
        var minutes = now.getMinutes();
        if (minutes < 10) minutes = "0" + minutes;
        var lastCheckTime = "(" + hours + ":" + minutes + ")";

        try {
          if (!result) {
            gYm_notifier.ym_log("no mail");
            gYm_notifier.alertUser(0,username,lastCheckTime);
          }
          else {
            gYm_notifier.ym_log("new mail: " + result + " messages");
          
            // Alert the user if needed
            gYm_notifier.alertUser(result,username,lastCheckTime);
   
            //  Allow user to override
            // TBD MULTI-USER SUPPORT
            var userInfo = gYm_notifier.prefs.loadUser("", false);
            if (userInfo && !userInfo.overrideURL) {
              delete userInfo;
              gYm_notifier.prefs.saveUserMailURL(gYm_notifier.mailURL);
            }
            userInfo = null;
          }
        }
        catch(ex) {
          gYm_notifier.ym_logError(ex, 1);
        }
      }
      else if (gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_ALT_CHECK) && (!gYm_notifier.useAltCheck)) {
        // something went wrong, use the alternate server instead and try again
        gYm_notifier.ym_log("Switching to alternate server");
        gYm_notifier.useAltCheck = true;    
        this.sendRetry2Id = setTimeout("gYm_notifier.checkMail();", 2000);
      }
      else {
        gYm_notifier.ym_log("Bad status, showing error");
        var observerService = Components.classes["@mozilla.org/observer-service;1"].
                              getService(Components.interfaces.nsIObserverService);
        observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
      }
    }
  }
  // If xmlHttpReq is ready, send request
  if (this.xmlHttpReq.readyState == 1) {
    // if retrying clear pending retry
    if (this.sendRetry2Id) {
      clearTimeout( this.sendRetry2Id );   // Clear pending retry  
      this.sendRetry2Id = null;
    }    	
    this.xmlHttpReq.send(null);
  }
  // if it is not ready try again in 2 seconds
  else {
    this.ym_log("Checkmail readyState = " + this.xmlHttpReq.readyState + ", retrying in 2 seconds.");
    this.xmlHttpReq.abort();
    this.sendRetry2Id = setTimeout("gYm_notifier.checkMail();", 2000);
  }
}

// Cookie functions to determine if user is logged in or not
// This will not tell what username user is logged in under though
//
// Returns:
//   0 if no user logged in
//   1 if logged in as different user
//   2 if logged in as correct user for browser session, but we want to verify login
//   3 if permanently logged in logged in as correct user, but we want to verify login 
//   4 if logged in as correct user for browser session
//   5 if permanently logged in logged in as correct user 
//
ym_notifier.prototype.loginState = function(username) {

  var userInfo = this.prefs.loadUser(username, false);
  
  // set correct URLs if there is an actual username
  if (userInfo) this.findYahooURL(userInfo.userName);

  // 'p' and 'l' will never change for a username, 'n' will change if password changes
	
  var cookie = this.getCookie('.' + this.cookieDomain, 'Y');
  if (!cookie) return 0;

  var value = cookie.value;
  if (!value) return 0;

  var cookieData = this.decodeQueryString(value);
  if (!cookieData) return 0;
  if (!cookieData['l'] || cookieData['l'] == '') return 0;

  var rc;
  if (userInfo && userInfo.cookie) {
    var userCookie = this.decodeQueryString(userInfo.cookie)
    if (userCookie && (cookieData['l'] == userCookie['l']) && (cookieData['p'] == userCookie['p'])) {
      if (cookieData['n'] == userCookie['n']) {
        if (userInfo.useCookie) {
          if ((cookieData['np'] || 0) == 1) rc = 4;
          else rc = 5;
        }
        else {
          if ((cookieData['np'] || 0) == 1) rc = 2;
          else rc = 3;
        }
      }
      else {
      	// n's aren't equal at this point which means password was changed so clear stored cookie
      	// and return 1
        this.ym_log(username + " changed password, clearing stored cookie.");
        this.prefs.saveUserCookie("", username);
      	rc = 1;
      }
    }
    else rc = 1;
  }
  else rc = 1;
  	
  if (userInfo) delete userInfo;
  return rc;
}

ym_notifier.prototype.decodeQueryString = function(qs) {
  qs = qs.replace(/%26/g, '&');
  qs = qs.replace(/%3D/g, '=');
  qs = qs.replace(/%25/g, '%');
  qs = qs.replace(/%3F/g, '?');
  var queryHash = new Array();
  var pairs = qs.split('&');
  for(var i = 0; i < pairs.length; i++) {
    var split = pairs[i].split('=');
    var key = split.shift();
    var val = split.join('=');
    queryHash[key] = val;
  }
  return queryHash;
}

ym_notifier.prototype.getCookie = function(domain, name) {
  var cmClass = "@mozilla.org/cookiemanager;1";
  var cms = Components.classes[cmClass].getService();
  var cm = cms.QueryInterface(Components.interfaces.nsICookieManager);
  var enumerator = cm.enumerator;
  var cIface = Components.interfaces.nsICookie;
  var domainRegex = new RegExp(domain.replace('\.', '\\.')+'$');
  while (enumerator.hasMoreElements()) {
    var nextCookie = enumerator.getNext().QueryInterface(cIface);
    if (nextCookie.host.match(domainRegex)) {
      if (nextCookie.name == name) return nextCookie;
    }
  }
  return null;
}

/*
 * alert the user if needed depending on the preferences
 */
ym_notifier.prototype.alertUser = function(nbMessages, username, lastCheckTime) {
  var prefs = this.prefs;
  
  var observerService = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
  observerService.notifyObservers(null,"ym-gui-change",nbMessages + " " + username + " " + lastCheckTime);

  if (nbMessages > 0) {
   	
    // Check to see if new mail count has changed to see if there really is new mail
    var trueNewMail = false;
    if (this.newMailCount != nbMessages) trueNewMail = true;
    this.newMailCount = nbMessages;
   	
    if ((prefs.getPref(prefs.PREF_SOUND) == prefs.CONST_REPEAT) ||
        ((prefs.getPref(prefs.PREF_SOUND) == prefs.CONST_ONCE) && trueNewMail)) {
      var soundUrl = decodeURIComponent(prefs.getPref(prefs.PREF_SOUNDFILE));
      var soundOk = false;

      // store sound instance in main object since sound will stop playing
      // if instance is destroyed.
      try {
      	if (!this.sound) this.sound = Components.classes["@mozilla.org/sound;1"].
      	                              createInstance(Components.interfaces.nsISound);
      	soundOK = true;
      }
      catch (e) {}
      
      if (soundOK) {

        var failed = true;
        var FileFactory = null;
        var soundFile = null;
        var aURI = false;

        // play default system mail sound if nothing specified
        if (soundUrl=="") {
          this.sound.playSystemSound("_moz_mailbeep"); 
          failed = true;
        }
        // if a URL use that
        else if (soundUrl.indexOf("://") != -1) {
          soundFile = soundUrl;
          aURI = true;
        }
        // must be a filename (local or absolute)
        else {
          failed = true;  // default to fail
          try {
            FileFactory = new Components.Constructor('@mozilla.org/file/local;1', 
                                                     'nsILocalFile', 'initWithPath');
            soundFile = new FileFactory(soundUrl);
            failed = false;
          } 
          catch (e) {}

          // if fail try to use a relative path in the chrome directory
          if (failed) {
            try {
              // get user chrome directory ( see http://kb.mozillazine.org/File_IO#Getting_special_files )
              var file = Components.classes["@mozilla.org/file/directory_service;1"]
                                   .getService(Components.interfaces.nsIProperties)
                                   .get("UChrm", Components.interfaces.nsIFile);
              file.append(soundUrl);
              FileFactory = new Components.Constructor('@mozilla.org/file/local;1', 
                                                       'nsILocalFile', 'initWithFile');
              soundFile = new FileFactory(file);
              failed = false;
            } 
            catch(ex) { 
              this.ym_logError(ex, 1);
            }
          }
        }
        if (!failed) {
          try {
            var soundURI = null;
            var nsIIOService = Components.classes['@mozilla.org/network/io-service;1'].
                               getService(Components.interfaces.nsIIOService);
            if (aURI) soundURI = nsIIOService.newURI(soundFile,null,null);
            else soundURI =  nsIIOService.newFileURI(soundFile);
            this.sound.play(soundURI);
          } 
          catch(ex) { 
            this.ym_logError(ex, 1);
          }
        }
      }
    }

    // TBD MULTI-USER SUPPORT
    var titleText = this.getString("new") + " Yahoo! Mail (" + 
                    prefs.getPref(prefs.PREF_DEFAULT_USER) + ")";
    var textStr = this.getString("youhave") +" " + nbMessages + " ";
    if (nbMessages == 1) textStr = textStr + this.getString("message");
    else textStr = textStr + this.getString("messages");

    if ((prefs.getPref(prefs.PREF_SHOW_ALERTS) == prefs.CONST_REPEAT) ||
        ((prefs.getPref(prefs.PREF_SHOW_ALERTS) == prefs.CONST_ONCE) && trueNewMail)) {
      try {
        var grn = Components.classes["@growl.info/notifications;1"].
                  getService(Components.interfaces.grINotifications);
        grn.sendNotification("Yahoo Mail Notifier", "chrome://ym_notifier/skin/newmail.png", 
        	                 titleText, textStr, "", this);
      }
      catch (ex) {
        try {
          var alerts = Components.classes["@mozilla.org/alerts-service;1"].
                       getService(Components.interfaces.nsIAlertsService);
          alerts.showAlertNotification("chrome://ym_notifier/skin/newmail.png", 
                                       titleText, textStr, true, "", this);
        } 
        catch(ex) { 
          this.ym_logError(ex, 1);
        }
      }
    }

    if ((prefs.getPref(prefs.PREF_SHOW_POPUP) == prefs.CONST_REPEAT) ||
        ((prefs.getPref(prefs.PREF_SHOW_POPUP) == prefs.CONST_ONCE) && trueNewMail)) {
      alert (titleText + "\n" + textStr);
    }
  }
  else {
    gYm_notifier.newMailCount = 0;
  }
}

/*
 * Observer functions
 */  
ym_notifier.prototype.observe = function(aSubject, aTopic, aData) {
	
  this.ym_log("Observer callback - aSubject : " + aSubject + ", aTopic : " +
              aTopic + ", aData : " + aData);
	            
  switch (aTopic) {
    // alert went away by itself
    case "alertfinished":
      break;

    // user clicked alert                   
    case "alertclickcallback":
      this.clickAlert();
      break;

    // user logged in or out or change of number of new messages
    case "ym-gui-change":
      this.updateGUI(aData);
      break;

    // preferences changed
    case "nsPref:changed":

      switch(aData) {

        // checking interval change
        case this.prefs.PREF_INTERVAL: 
        
          if (this.timer || this.prefs.getPref(this.prefs.PREF_AUTO_CHECK))
            this.startChecking();
          break;
        
        case this.prefs.PREF_STATUSBAR:
        
          var prefs = gYm_notifier.prefs;
          var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
          if (myStatusbar) {
            var showStatusbar = ((prefs.getPref(prefs.PREF_STATUSBAR) == prefs.CONST_ALWAYS) ||
                                 ((prefs.getPref(prefs.PREF_STATUSBAR) == prefs.CONST_ON_NEW_MAIL) && 
                                  (myStatusbar.getAttribute("newMail") == true)));
            myStatusbar.setAttribute("collapsed", !showStatusbar);
          }
          break;
      }
      break;

    // timer periodic call
    case "timer-callback":

      this.checkMail();
      break;
  }
}

// nsIAlertsService - nsIAlertListener was deprecated and replaced with nsIObserver
//                    This is to maintain 1.7 compatability.
ym_notifier.prototype.onAlertFinished = function(aCookie) {}
ym_notifier.prototype.onAlertClickCallback = function(aCookie) {
   this.clickAlert();
}

/*
 * Update statusbar and toolbar
 */
ym_notifier.prototype.updateGUI = function(aData) {
  var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
  var myToolbar = document.getElementById("ym-notifier-toolbar-item");
  var myLogout = document.getElementById("ym-notifier-context-menu-logout");
  var myMenu = document.getElementById("ym-notifier-context");

  aData = aData.split(" ");
  
  var loginState = this.loginState(aData[1]);
  var checkerror = (aData[0] == "check-error");
  var justloggedin = ((aData[0] == "login") && (loginState > 0));
  var loggedin = ((aData[0] != "logout") && ((loginState > 1) || justloggedin));
  var mailChange = ((aData[0] != "login") && (aData[0] != "logout"));
  var newMail = (aData[0] > 0); 
  var myTooltip;
  
  this.ym_log("loginState : " + loginState + ", checkerror : " + checkerror +
              ", loggedin : " + loggedin + ", justloggedin : " + justloggedin +
              ", mailChange : " + mailChange + ", newMail : " + newMail + 
              ", aData[0] : " + aData[0] + ", aData[1] : " + aData[1] + 
              ", aData[2] : " + aData[2]);


  // errors take highest preference
  if (checkerror) {
    myTooltip = aData[1] + ": " + this.getString("checkerror");
    newMail = false;
    aData[0] = "";
    mailChange = true;
  }
  // logging out takes preference over everything except errors
  else if (!loggedin) {
    this.stopChecking();
    myTooltip = this.getString("nologin");
    newMail = false;
    mailChange = true;
    aData[0] = "";
    this.newMailCount = 0;
  }
  else {
    // if default user just logged in  // TBD MULTI-USER SUPPORT
    if (justloggedin &&
       (aData[1] == this.prefs.getPref(this.prefs.PREF_DEFAULT_USER))) {

      // set correct URLs
      this.findYahooURL(aData[1]);
      // store updated login and name part of the login cookie
      var cookie = this.getCookie('.' + this.cookieDomain, 'Y');
      this.prefs.saveUserCookie(cookie.value);
//      var cookieData = this.decodeQueryString(cookie.value);
//      this.prefs.saveUserCookie('l=' + cookieData['l'] + "&n=" + cookieData['n'] +
//                                '&p=' + cookieData['p'], aData[1]);

      // only start checking on login if not already checking
      if (!this.timer) this.startChecking();
      else {
      	// Workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=174320 in older browsers
        if (this.oldBrowser) window.setTimeout("gYm_notifier.checkMail();", 0)
        else gYm_notifier.checkMail();
      }
    }

    if (newMail) {
      myTooltip = this.getString("youhave") + " " + aData[0] + " ";
      if (aData[0] == 1) myTooltip = myTooltip + this.getString("message");
      else myTooltip = myTooltip + this.getString("messages");
    }
    else {
      myTooltip = this.getString("nomail");
    }  
  }

  // if just checked for mail, add time to tooltip otherwise grab last checked 
  // time from statusbar attribute
  if (aData[2] && aData[2]!="undefined") myTooltip = myTooltip + " " + aData[2];
  else if (myStatusbar && loggedin) 
    myTooltip = myTooltip + " " + myStatusbar.getAttribute("lastCheckTime");
      
  if (myStatusbar) {
    myStatusbar.setAttribute("logged-in", loggedin);
    myStatusbar.setAttribute("error", checkerror);
    myStatusbar.setAttribute("checking", (aData[2] && aData[2]!="undefined"));
    if (mailChange) {
      myStatusbar.setAttribute("tooltiptext", myTooltip);
      myStatusbar.setAttribute("newMail", newMail);

      // do not change current count on error
      if (!checkerror) {
        myStatusbar.setAttribute("label", aData[0]);
        if (loggedin) myStatusbar.setAttribute("lastCheckTime", aData[2]);
        else myStatusbar.setAttribute("lastCheckTime", "");
      }

      // show or hide status bar if setting is set to only show on new mail
      if (this.prefs.getPref(this.prefs.PREF_STATUSBAR) == this.prefs.CONST_ON_NEW_MAIL)
        myStatusbar.setAttribute("collapsed", !newMail);
    }
  }
  if (myToolbar) {
    myToolbar.setAttribute("logged-in", loggedin);
    myToolbar.setAttribute("error", checkerror);
    if (mailChange) {
      myToolbar.setAttribute("tooltiptext", myTooltip);
      myToolbar.setAttribute("newMail", newMail);

      // do not change current count on error
      if (!checkerror) myToolbar.setAttribute("unread", aData[0]);
    }
  }
  if (myLogout) myLogout.setAttribute("disabled", !loggedin);
  
  // hide the context popup menu if it is displayed
  if (myMenu) myMenu.hidePopup();
  
  // clear aData
  aData = null;
}	

/*
 * Go to mail url if user clicks alert window.  Since user can only left click an alert, 
 * this will only open in a new tab or window based off the single click (or double click
 * if single click is not set or set to same tab) preference. This will never overwrite 
 * the current tab since user might not want to do so.
 */
ym_notifier.prototype.clickAlert = function() {
  //If user clicks alert on mail in new tab
  try {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
               getService(Components.interfaces.nsIWindowMediator);
    var top = wm.getMostRecentWindow("navigator:browser");
    if (!top)
      this.gotoYahooMail(this.prefs.CONST_NEWWIN, null);
    else {
      var option = this.prefs.getPref(this.prefs.PREF_CLICK);
      if ((option == this.prefs.CONST_NOTHING) || (option == this.prefs.CONST_SAMETAB) ||
          (option >= this.prefs.CONST_CHECKMAIL))
        option = this.prefs.getPref(this.prefs.PREF_DOUBLECLICK);
      if ((option == this.prefs.CONST_NOTHING) || (option == this.prefs.CONST_SAMETAB) ||
          (option >= this.prefs.CONST_CHECKMAIL))
        option = this.prefs.CONST_FOCUSTAB;
      this.gotoYahooMail(option, top.document.getElementById("content"));
    }
  }
  catch(ex) {
    // There are no browser windows open and therefore no component services available (no preferences)
    // so open a new window and go to this.mailURL as a last ditch effort
    window.openDialog("chrome://browser/content/browser.xul", "_blank",
                                "chrome,all,dialog=no", this.mailURL, null, null);
  }
}

/* 
 * handles single, double and middle button mouse clicks
 */
ym_notifier.prototype.handleClick = function(event) {
  var prefs = gYm_notifier.prefs;
	
  // ignore clicks with ctrl or alt key pressed
  if ((event.ctrlKey)||(event.altKey)) return false;

  // for our purposes a double click is 2 single clicks
  if (event.type == "click") {
    var eShiftKey = ((!(event.ctrlKey)||(event.metaKey)) && (!event.altKey) && (event.shiftKey));
    var midMouse = ((event.button == 1) || ((event.button == 0) && (event.shiftKey)))
               
    // if not a left click, middle mouse click or shift click return
    if ((event.button != 0) && !midMouse) return false;
         
    if (midMouse) { 
      this.gotoYahooMail(prefs.getPref(prefs.PREF_MIDDLECLICK), document.getElementById("content"));
    }
    else {
      var dcTime = prefs.getPref(prefs.PREF_DOUBLECLICK_TIME);

      // If we've just had a click then this is a double click
      if (this.mouseId) {
        clearTimeout( this.mouseId );   // Clear pending Click  
        this.mouseId = null;
        this.gotoYahooMail(prefs.getPref(prefs.PREF_DOUBLECLICK), document.getElementById("content"));
      }
      // first click, might be start of double click so wait a few ms before processing
      else {
        var d = new Date();
        var now = d.getTime();

        this.mouseEvtTime = now;
        this.mouseId = setTimeout("gYm_notifier.doClick(gYm_notifier.prefs.getPref(gYm_notifier.prefs.PREF_CLICK));", dcTime);
      }
    }
  }
  return false;
}

/* 
 * this handles single clicks that aren't actually double clicks
 */
ym_notifier.prototype.doClick = function(clickPref) {
  this.mouseId = null;
  this.gotoYahooMail(clickPref, document.getElementById("content"));
  return false;
}
   	

/* 
 * opens Yahoo! Mail based off of user clicking preferences
 */
ym_notifier.prototype.gotoYahooMail = function(option, tBrowser) {
  // read stored mailurl or use default if none stored
  var prefs = this.prefs;
  var userInfo = this.prefs.loadUser("", false);           // TBD MULTI-USER SUPPORT
  
  var mailURL = this.mailURL;
  if (userInfo) {
     // find the correct default URL
    this.findYahooURL(userInfo.userName);
    if (userInfo.overrideURL && (userInfo.mailURL != "")) mailURL = userInfo.mailURL;
    delete userInfo;
  }
  
  // if option is greater than CONST_CHECKMAIL then user wants to check mail on click
  if (option >= prefs.CONST_CHECKMAIL) this.manualCheckMail();
  else {
  
    // Is current tab blank.  Different methods for determining depending on whether browser is
    // FF 1.5 or FF 1.0 based
    var currBlank =  (tBrowser && 
                      (tBrowser.mCurrentTab.linkedBrowser && 
                       (tBrowser.mCurrentTab.linkedBrowser.contentDocument.location == "about:blank")) ||
                      (!tBrowser.mCurrentTab.linkedBrowser && 
                       (tBrowser.mCurrentTab.label == "(Untitled)")));
                       
    // If using FF 1.5 or greater, try and find if Yahoo Mail is open in a tab and just switch there.
    if (tBrowser && tBrowser.mCurrentTab.linkedBrowser) {
      for (var i=0; i<tBrowser.mTabs.length; i++) {
        var tablocation = String(tBrowser.mTabs[i].linkedBrowser.contentDocument.location).toLowerCase();
        // find a Yahoo Mail tab that isn't being used for composing
        if ((tablocation.indexOf("mail.yahoo") != -1) && (tablocation.indexOf("/ym/compose") == -1)) {
          tBrowser.selectedTab = tBrowser.mTabs[i];
          // if not Yahoo Mail Classic don't reload
          if (tablocation.indexOf("/dc/") != -1) {
            option = prefs.CONST_NOTHING;
          }
          else {
            option = prefs.CONST_SAMETAB;
          }
          break;
        }
      }
    }
     
    switch (option) {
      case prefs.CONST_NOTHING:
        return;
        break;
   	        
      case prefs.CONST_SAMETAB:
        tBrowser.loadURI(mailURL);
        break;
   	           
      case prefs.CONST_NEWTAB:
        if (currBlank) tBrowser.loadURI(mailURL);
        else tBrowser.addTab(mailURL);
        break;
   	           
      case prefs.CONST_FOCUSTAB:
        if (currBlank) tBrowser.loadURI(mailURL);
        else {
          var tab = tBrowser.addTab(mailURL);
          tBrowser.selectedTab = tab;
        }
        break;
   	           
      case prefs.CONST_NEWWIN:
        window.open(mailURL, "", "");
        break;
    }

    // set icons and text to no new mail if checking for mail
    var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
    if (myStatusbar && myStatusbar.getAttribute("checking")) this.alertUser(0);
  }
}

/*
 * Open the preferences window
 */ 
ym_notifier.prototype.openPrefWindow = function() {
  var prefWindow = window.openDialog('chrome://ym_notifier/content/ym_prefs.xul', 'ym_pref_window', 
                    'close=no, dialog, chrome, modal, centerscreen');
  prefWindow.focus();
}

/*
 * Open the login window
 */ 
ym_notifier.prototype.openLoginWindow = function() {
  var loginWindow = window.openDialog('chrome://ym_notifier/content/ym_login.xul', 'ym_login_window', 
                    'close=no, dialog, chrome, modal, centerscreen');
  loginWindow.focus();
}

/*
 * Get the localized strings
 */
ym_notifier.prototype.getString = function(stringName) {
  var strings = document.getElementById("ym-notifierStrings");
  return strings.getString(stringName);
}

/**
 * Utility to create an error message in the log without throwing an error.
 */
ym_notifier.prototype.ym_logError = function(e, force) {
  try { 
    if (force || this.prefs.getPref(this.prefs.PREF_LOG_ENABLED)) {
      var consoleService = Components.classes['@mozilla.org/consoleservice;1'].
                           getService(Components.interfaces.nsIConsoleService);

      var consoleError = Components.classes['@mozilla.org/scripterror;1'].
                         createInstance(Components.interfaces.nsIScriptError);

      consoleError.init(e.message, e.fileName, e.lineNumber, e.lineNumber,
                        e.columnNumber, 0, null);

      consoleService.logMessage(consoleError);
      this.ym_write_log(consoleError + "\n");
    }
  }
  catch (ex) {
    dump (ex + "\n")
  }
}

/*
 * Log info messages
 */
ym_notifier.prototype.ym_log = function(aMessage, force) {
  try {
    if (force || this.prefs.getPref(this.prefs.PREF_LOG_ENABLED)) {
      var consoleService = Components.classes['@mozilla.org/consoleservice;1'].
                           getService(Components.interfaces.nsIConsoleService);
      consoleService.logStringMessage("ym_notifier: " + aMessage);
      this.ym_write_log(aMessage + "\n");
    }
  }
  catch (ex) { 
    dump(ex + "\n"); 
  }
}

/*
 * Write to Log File
 */ 
ym_notifier.prototype.ym_write_log = function(aMessage) {
  try {
    if (!this.logFile) {
      this.logFile = Components.classes["@mozilla.org/file/directory_service;1"].
                     getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsILocalFile).clone();
		  this.logFile.append("Yahoo_Mail_Notifier_log.txt");
	  }
		
		var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		// ioFlags: write only, create file, append;  Permission: read/write owner
		stream.init(this.logFile, 0x02 | 0x08 | 0x10, 0600, 0);
		var cvstream = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
		cvstream.init(stream, "UTF-8", 0, Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
		
		cvstream.writeString(aMessage.replace(/[\n$]/g, this.mEOL));
		cvstream.flush();
		cvstream.close();
  }
  catch (ex) { 
    dump(ex + "\n"); 
  }
}

ym_notifier.prototype.getEOL = function() {
  return /win|os[\/_]?2/i.test(navigator.platform)?"\r\n":/mac/i.test(navigator.platform)?"\r":"\n";
}
 
/*
 * Uninstall procedure
 */
ym_notifier.prototype.uninstall = function() {
  var confirmUninstall = confirm(this.getString("confirmuninstall"));
	
  // user choose to cancel
  if (!confirmUninstall) return; // do nothing further
	
  // disable the "Uninstall" menuitem
  document.getElementById("ym-notifier-context-menu-uninstall").setAttribute("disabled", true); 
	
  var oExtUninstall = new ExtensionUninstall("urn:mozilla:package:ym_notifier");
  var success = oExtUninstall.uninstall();
  this.uninstalled(oExtUninstall);
}

/*
 * Output Result of Uninstall
 */
ym_notifier.prototype.uninstalled = function(oExtUninstall) {
  var message;
	
  if(oExtUninstall != null && oExtUninstall.hasError())
    message = this.getString("uninstallerror");
  else if(oExtUninstall != null && oExtUninstall.hasWarning())
    message = this.getString("uninstallwarning");
  else
    message = this.getString("uninstallsuccess");
	
  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                getService(Components.interfaces.nsIPromptService);
  var flags = prompts.BUTTON_TITLE_IS_STRING * prompts.BUTTON_POS_0 + 
              prompts.BUTTON_TITLE_IS_STRING * prompts.BUTTON_POS_1;
  var check = {value: false};
  var button = prompts.confirmEx(window, this.getString("uninstalldone"), message, flags, 
                                 this.getString("closebutton"), this.getString("viewlogbuton"), null, null, check);
  if (button) oExtUninstall.showLogDialog(true, self);
}

/*
 * Only display uninstall for Mozilla and SeaMonkey versions less than 2.0
 */ 
ym_notifier.prototype.showUninstall = function() {
  if ((!/(BonEcho|Minefield|Flock|Firefox|Netscape)/.test(navigator.userAgent)) &&
      (/(SeaMonkey\/(\d+))/.test(navigator.userAgent) && RegExp.$2 < 2)) {
    document.getElementById('ym-notifier-context-menu-uninstall').removeAttribute('hidden');
  }
}

var gYm_notifier = new ym_notifier();
window.addEventListener('load', gYm_notifier.initialize, false);
